'''
Simulation code for Murray JD et al. (2014) Cereb Cortex.
Author: John D. Murray (john.murray@yale.edu)
'''
from brian import *
from numpy.fft import rfft,irfft
import time
import numpy as np
from scipy.special import erf


### Load default parameters ###
execfile('parameters.py')

### Here you can change any parameters, such as NMDA onto I-cells (Gei_nmda) ####
suffix = 'disinhib' # set string attached to spike raster file names
Gei_nmda = (1. - 0.0325) * Gei_nmda

#suffix='control'
# Gei_nmda = (1. - 0.) * Gei_nmda


### Begin setting up simulation ###

t0=time.clock()

circ_distance = lambda theta: min(mod(theta,360),mod(360.-theta,360)) # circular distance in degrees
# connectivity footprint
tmp   = np.sqrt(2*pi)*sigma_ee*erf(360.*0.5/np.sqrt(2.)/sigma_ee)/360.
Jm_ee = (1.-Jp_ee*tmp)/(1.-tmp)
weight_fn = lambda theta: (Jm_ee+(Jp_ee-Jm_ee)*np.exp(-0.5*circ_distance(theta)**2/sigma_ee**2))
weight_e = [weight_fn(i*360./NE) for i in xrange(NE)]
fweight = rfft(weight_e) # Fourier transform


current_clock = Clock(dt=dt_curr)
tc_stop       = tc_start + tc_dur
tdist_stop    = tdist_start + tdist_dur

# calculating the cue currents
currents = lambda i,j: np.exp(-0.5*circ_distance((i-j)*360./NE)**2/i_cue_width**2)
current_e=np.zeros(NE)
j_cue = i_cue_ang*NE/360.
for i in xrange(NE):
    current_e[i]=currents(i,j_cue)

# calculating the distractor currents
currents_d = lambda i,j: np.exp(-0.5*circ_distance((i-j)*360./NE)**2/i_dist_width**2)
current_e_d=zeros(NE)
j = i_dist_ang*NE/360.
for i in xrange(NE):
    current_e_d[i]=currents_d(i,j)


# Define equations

eqs_e = '''
dv/dt = (-gl_e*(v-El_e) - g_ext_e*s_ext*(v-E_ampa) - Gee_nmda*s_tot*(v-E_nmda)/(1+b*exp(-a*v)) + - Gie*s_gaba*(v-E_gaba) + i_e)/Cm_e: volt
ds_ext/dt = -s_ext/t_ampa : 1
ds_gaba/dt = -s_gaba/t_gaba : 1
ds_nmda/dt = -s_nmda/t_nmda + alpha*x*(1-s_nmda) : 1
dx/dt = -x / t_x : 1
s_tot : 1
i_e : amp
'''

eqs_i = '''
dv/dt = (-gl_i*(v-El_i) - g_ext_i*s_ext*(v-E_ampa) - Gei_nmda*(v-E_nmda)*s_tot/(1+b*exp(-a*v)) - Gii*s_gaba*(v-E_gaba) + i_e)/Cm_i: volt
ds_ext/dt = -s_ext / t_ampa : 1
ds_rec_ampa/dt = -s_rec_ampa / t_ampa : 1
ds_gaba/dt = -s_gaba / t_gaba : 1
s_tot : 1
i_e : amp
'''

# Define neuron groups
Pe = NeuronGroup(NE, eqs_e, threshold=Vth_e, reset=Vr_e, refractory=tr_e, clock=simulation_clock, order=2, freeze=True)
Pe.v = El_e
Pe.s_ext = 0
Pe.s_gaba = 0
Pe.s_nmda = 0
Pe.x = 0
Pe.i_e = 0

Pi = NeuronGroup(NI, eqs_i, threshold=Vth_i, reset=Vr_i, refractory=tr_i, clock=simulation_clock, order=2, freeze=True)
Pi.v = El_i
Pi.s_ext = 0
Pi.s_gaba = 0
Pi.i_e = 0

# Define external Poisson input
PGe  = PoissonGroup(NE, fext_e, clock=simulation_clock) # to E-cells
PGi  = PoissonGroup(NI, fext_i, clock=simulation_clock) # to I-cells

# Define connections
selfnmda = IdentityConnection(Pe,Pe,'x',weight=1.0) # Update of NMDA gating variable of E-cells (pre-synaptic)

Cpe = IdentityConnection(PGe, Pe, 's_ext', weight=1.0) # External AMPA drive to E-cells (post-synaptic)
Cpi = IdentityConnection(PGi, Pi, 's_ext', weight=1.0) # External AMPA drive to I-cells (post-synaptic)

Cie = Connection(Pi, Pe, 's_gaba', weight=1.0) # Recurrent GABA I->E (post-synaptic)
Cii = Connection(Pi, Pi, 's_gaba', weight=1.0) # Recurrent GABA I->I (post-synaptic)

# Calculate NMDA contributions
@network_operation(simulation_clock, when='start')
def update_nmda(simulation_clock):
    fsnmda = rfft(Pe.s_nmda)
    fstot = fsnmda*fweight
    Pe.s_tot = irfft(fstot, NE)
    Pi.s_tot = fsnmda[0]

# Update stimulus input
@network_operation(current_clock, when='start')
def update_currents(current_clock):
    c_time = current_clock.t
    if (c_time>tc_start and c_time<tc_stop):
        Pe.i_e = i_cue_amp*current_e
    elif (c_time>tdist_start and c_time<tdist_stop):
        Pe.i_e = i_dist_amp*current_e_d
    else:
        Pe.i_e = 0

mon_spikes_e = FileSpikeMonitor(Pe,'spikes_e_'+suffix+'.dat',record=True)
mon_spikes_i = FileSpikeMonitor(Pi,'spikes_i_'+suffix+'.dat',record=True)


# Run simulation
run(simtime, report='text')
